@@ -25,6 +25,8 @@ VERSION='0.3.2'
 # ----------------------------------------------------------------
 # Configurable items:
 
+shopt -s extglob
+
 # Root directory of your virtual servers (probably shouldn't change this)
 VROOTDIR="/vservers"
 
@@ -35,11 +37,9 @@ INSTALL_PACKAGES="less,dnsutils,wget,bzip2,ssh,rsync,libssl0.9.6,libdns5,libisc4
 
 # Packages installed from within the vserver after it has
 # been setup (seperated by "," commas)
-#EXTRA_PACKAGES="emacs20,lynx-ssl"
 EXTRA_PACKAGES="nfsbooted"
 
 # Packages to remove from the base defaults (seperated by "," commas)
-#REMOVE_PACKAGES="dhcp-client,lilo,makedev,pcmcia-cs,ppp,pppconfig,pppoe,pppoeconf,setserial,syslinux,nano,fdutils,iptables,libpcap0,pciutils"
 REMOVE_PACKAGES="lilo,makedev,nfs-kernel-server,lpr"
 
 # sysvinit services relating to hardware access to remove
@@ -49,8 +49,8 @@ REMOVE_PACKAGES="lilo,makedev,nfs-kernel-server,lpr"
 # otherwise dpkg will detect whether we are i386/powerpc/sparc/etc
 ARCH=""
 
-# Which debian distribution (warning: this has only been tested with woody)
-DIST="woody"
+# Which debian distribution (warning: this has only been tested with woody and sarge)
+DIST="sarge"
 
 # Local or nearest location of a debian mirror (must include the `/debian')
 MIRROR="http://ftp.uk.debian.org/debian"
@@ -101,7 +101,7 @@ Options:
 				(autodetected on Debian host if dpkg available)
 	--copy-vreboot		install "vreboot/vshutdown/vhalt"
 	--no-copy-vreboot	don't install "vreboot/vshutdown/vhalt"
-	--dist			defaults to "woody", passed to debootstrap.
+	--dist			defaults to "sarge", passed to debootstrap.
 	--fakeinit		use "/sbin/init" to boot vserver
 	--conffile		extra configuration file to load.
 	--interface		interface for IP addresses (if not "eth0")
@@ -185,13 +185,13 @@ parse_args ()
 			case "$2" in
 			    [a-z]*)
 				DIST="$2"
-				if [ "woody" != "$2" ]; then
-				    echo "${0##*/} warning: I only know how to do \"woody\", be careful!" 1>&2
+				if [ "sarge" != "$2" ] && [ "woody" != "$2" ] ; then
+				    echo "${0##*/} warning: I only know how to do \"woody\" and \"sarge\", be careful!" 1>&2
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a Debian distribution" 1>&2
-				echo 'e.g. "woody"' 1>&2
+				echo 'e.g. "woody" or "sarge"' 1>&2
 				exit 1
 				;;
 			esac
@@ -217,7 +217,7 @@ parse_args ()
 			;;
 		    --hostname)
 			case "$2" in
-			    [a-z]*[a-z0-9])
+			    *([a-z0-9])[a-z]*([a-z0-9]))
 				VHOST="$2"
 				;;
 			    *)
@@ -442,6 +442,11 @@ if [ -f /etc/apt/apt.conf ]; then
     cp /etc/apt/apt.conf $VROOTDIR/$VHOST/etc/apt/apt.conf
 fi
 
+# support the apt.conf.d directories
+if [ -d /etc/apt/apt.conf.d ] ; then
+    cp -a /etc/apt/apt.conf.d/* $VROOTDIR/$VHOST/etc/apt/apt.conf.d/ > /dev/null 2>&1
+fi
+
 # This old uname was putting the name of the *host* into the vserver motd
 # uname -a > $VROOTDIR/$VHOST/etc/motd
 
