@@ -1,9 +1,10 @@
 #!/bin/bash
-VERSION='0.3.2'
 #
-# Copyright (C) 2002       Mark Lawrence <nomad@null.net>
-# Copyright (C) 2002       Paul Sladen   <vserver@paul.sladen.org>
 # Copyright (C) 2002-2005  Ola Lundqvist <opal@debian.org>
+# Copyright (C) 2005       Thomas Gelf <thomas@gelf.net>
+# Copyright (C) 2005       Daniel Hermann <hermann@tkm.physik.uni-karlsruhe.de>
+# Copyright (C) 2002       Paul Sladen   <vserver@paul.sladen.org>
+# Copyright (C) 2002       Mark Lawrence <nomad@null.net>
 #
 # This is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -25,17 +26,18 @@ VERSION='0.3.2'
 # ----------------------------------------------------------------
 # Configurable items:
 
+shopt -s extglob
+
 # Root directory of your virtual servers (probably shouldn't change this)
-VROOTDIR="/vservers"
+# Support for 0.30.203 and above of util-vserver
+VROOTDIR="/etc/vservers/.defaults/vdirbase"
 
 # Packages to install in addition to the base defaults
 # MUST INCLUDE ALL DEPENDENCIES (seperated by "," commas)
-#INSTALL_PACKAGES="less,dnsutils,wget,bzip2,ssh,rsync,libssl0.9.6,liblwres1,bind9-host,zlib1g,libbz2-1.0,traceroute"
 INSTALL_PACKAGES=""
 
 # Packages installed from within the vserver after it has
 # been setup (seperated by "," commas)
-#EXTRA_PACKAGES="emacs20,lynx-ssl"
 EXTRA_PACKAGES=""
 
 # Packages to remove from the base defaults (seperated by "," commas)
@@ -43,7 +45,7 @@ EXTRA_PACKAGES=""
 REMOVE_PACKAGES="sparc-utils,dhcp-client,lilo,makedev,pcmcia-cs,ppp,pppconfig,pppoe,pppoeconf,setserial,syslinux,fdutils,libpcap0,iptables,pciutils"
 
 # sysvinit services relating to hardware access to remove
-REMOVE_LINKS="klogd hwclock.sh setserial urandom networking umountfs halt reboot"
+REMOVE_LINKS="klogd hwclock.sh setserial urandom networking umountfs halt reboot mountvirtfs mountall.sh mountnfs.sh ifupdown"
 
 # Post installation script
 POST_INSTALL_SCRIPT=""
@@ -62,22 +64,13 @@ MIRROR="http://ftp.uk.debian.org/debian"
 # Default network interface for vservers:
 INTERFACE="eth0"
 
-# Copy vreboot/vhalt/vshutdown utility into /usr/local/sbin/
-COPY_VREBOOT="true"
-
-if [ -r /etc/vservers/util-vserver-vars ] ; then
-    . /etc/vservers/util-vserver-vars
-fi
+# Package caching
+PKGCACHE=0
 
 if [ -r /etc/vservers/newvserver-vars ] ; then
     . /etc/vservers/newvserver-vars    
 fi
 
-# Support for 0.30.203 and above of util-vserver
-if [ -n "$DEFAULT_VSERVERDIR" ] ; then
-    VROOTDIR="$DEFAULT_VSERVERDIR"
-fi
-
 # ----------------------------------------------------------------
 # Nothing from here on should need changing.
 # ----------------------------------------------------------------
@@ -95,28 +88,28 @@ EOF
 full_usage ()
 {
 	cat << EOF
-Usage:	${0##*/} [OPTIONS] --hostname x --domain y.z --ip 1.2.3.4
-Creates a new Debian vserver by downloading packages via HTTP/FTP
+Usage:	${0##*/} [OPTIONS] --hostname x --domain y.z --ip 1.2.3.4/24
+Creates a new Debian vserver by calling "vserver ... build"
 
 Options:
         -h, --help		this help
         -V, --version		copyright and version information
 	--arch			set target architecture (eg. --arch "i386")
 				(autodetected on Debian host if dpkg available)
-	--copy-vreboot		install "vreboot/vshutdown/vhalt"
-	--no-copy-vreboot	don't install "vreboot/vshutdown/vhalt"
 	--dist			defaults to "sarge", passed to debootstrap.
 	--fakeinit		use "/sbin/init" to boot vserver
 	--conffile		extra configuration file to load.
 	--interface		interface for IP addresses (if not "eth0")
 	--mirror		Debian HTTP/FTP mirror (including the /debian)
 	--sshkeys		copy pub-keys to "/root/.ssh/authorized_keys"
+	--pkgcache              Package caching.
 	-v, --verbose		show extra output during setup
 	--vsroot		location of "/vserver/" directory
 Required:
 	--hostname		hostname for new vserver (eg. "alpha")
 	--domain		dns domain for new vserver (eg. "example.com")
 	--ip			IPv4 address for new vserver
+				(syntax: --ip <ip>[/<netmask|prefixlength>])
 
 You can also set variables in /etc/vservers/newvserver-vars.
 
@@ -125,10 +118,11 @@ EOF
 full_version ()
 {
     cat << EOF
-${0##*/} version $VERSION
-Copyright (c) 2002      Mark Lawrence <nomad@null.net>
-Copyright (c) 2002      Paul Sladen   <vserver@paul.sladen.org>
+${0##*/}
 Copyright (c) 2002-2005 Ola Lundqvist <opal@debian.org>
+Copyright (c) 2005      Daniel Hermann <hermann@tkm.physik.uni-karlsruhe.de>
+Copyright (c) 2002      Paul Sladen   <vserver@paul.sladen.org>
+Copyright (c) 2002      Mark Lawrence <nomad@null.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -184,14 +178,6 @@ parse_args ()
                        fi
                        shift 2
                        ;;
-		    --copy-vreboot)
-			COPY_VREBOOT="true"
-			shift
-			;;
-		    --no-copy-vreboot)
-			COPY_VREBOOT=""
-			shift
-			;;
 		    --dist)
 			case "$2" in
 			    [a-z]*)
@@ -202,7 +188,7 @@ parse_args ()
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a Debian distribution" 1>&2
-				echo 'e.g. "sarge"' 1>&2
+				echo 'e.g. "woody" or "sarge"' 1>&2
 				exit 1
 				;;
 			esac
@@ -226,9 +212,13 @@ parse_args ()
 			FAKEINIT=" fakeinit"
 			shift
 			;;
+		    --pkgcache)
+		        PKGCACHE=1
+			shift
+		        ;;
 		    --hostname)
 			case "$2" in
-			    [a-z]*[a-z0-9])
+			    *([a-z0-9])*([a-z0-9_-])[a-z]*([a-z0-9_-])*([a-z0-9]))
 				VHOST="$2"
 				;;
 			    *)
@@ -257,6 +247,18 @@ parse_args ()
 			case "$2" in
 			    [0-9]*.[0-9]*.[0-9]*.[0-9]*)
 				IP="$2"
+				# get netmask
+				IP_NETMASK=${IP##*/}
+				test "$IP_NETMASK" != "$1" || IP_NETMASK=
+				IP_ADDR=${IP%%/${IP_NETMASK}}
+				# if IP_NETMASK contains no ".", it is
+				# interpreted as IP_PREFIX 
+				IP_PREFIX=
+				test "${IP_NETMASK%%.*}" != "${IP_NETMASK}" \
+				  || {
+				    IP_PREFIX=$IP_NETMASK
+				    IP_NETMASK=
+				}
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a single IPv4  e.g. \"192.168.100.1\"" 1>&2
@@ -340,17 +342,6 @@ VROOTDIR="${VROOTDIR%/}"
 # Dist specific issues
 ##############################################################################
 
-
-# Check we've got debootstrap available
-if [ ! -x /usr/sbin/debootstrap ]; then
-  cat << EOF 1>&2
-${0##*/}: Requires the debootstrap package to bootstrap Debian
-  Debian Host:	apt-get install debootstrap
-  RPM Host:	rpm -i http://people.debian.org/~blade/install/debootstrap/debootstrap-0.1.17.3-2.i386.rpm
-EOF
-  exit 1
-fi
-
 if ! cat /proc/self/status | grep '^s_context:[^0-9]0$'; then
     echo "${0##*/} error:"
     echo "	Must be run from the host server (security context 0)" 1>&2
@@ -364,19 +355,26 @@ if [ -x /usr/bin/id ] && [ `id -u` -ne 0 ]; then
   exit 1
 fi
 
-# check for /vserver/$VHOST/etc/ incase we are on pre-mounted LVM partition
-# (used to just check for "/vserver/$VHOST/" existing
-if [ -d "$VROOTDIR/$VHOST/etc/" -o -f "/etc/vservers/$VHOST.conf" ] ; then
-  cat << EOF 1>&2
-${0##*/} error: Virtual Server "$VHOST" appears to already exist
-  check "/etc/vservers/$VHOST.conf" or "/vservers/$VHOST/etc/";
-EOF
-  exit 1
-fi
+aptcleanup() {
+    apt-get autoclean -o Dir::Cache::Archives="$1"
+    OLD=""
+    for P in $(find "$1" -name *.deb -printf "%f\n" -mindepth 1 -maxdepth 1 | sed -e "s/_.*//;") ; do
+	if [ "$P" == "$OLD" ] ; then
+	    rm -f "$1"/$P_* > /dev/null 2>&1
+	fi
+	OLD=$P
+    done
+}
 
 # This is used to keep a cache of the downloaded .deb packges for next install
-if [ -d "$VROOTDIR/ARCHIVES/$DIST" ]; then
+if [ -d "$VROOTDIR/ARCHIVES/$DIST" ] && [ $PKGCACHE -eq 1 ] ; then
     mkdir -p "$VROOTDIR/$VHOST/var/cache/apt/archives"
+    if [ ! -d "$VROOTDIR/ARCHIVES/$DIST/partial" ] ; then
+	# Needed to make apt-get autoclean work fine.
+	mkdir "$VROOTDIR/ARCHIVES/$DIST/partial"
+    fi
+    echo "Clean package cache."
+    aptcleanup "$VROOTDIR/ARCHIVES/$DIST"
     cp -a "$VROOTDIR/ARCHIVES/$DIST/"*.deb "$VROOTDIR/$VHOST/var/cache/apt/archives" > /dev/null 2>&1
 fi
 
@@ -385,12 +383,14 @@ if [ -n "$ARCH" ]; then
     ARCH_ARGUMENT="--arch $ARCH"
 fi
 
-# Fire off `debootstrap' to do all the hard work
-# like downloading and installing
-if ! /usr/sbin/debootstrap $ARCH_ARGUMENT \
-	"--include=$INSTALL_PACKAGES" "--exclude=$REMOVE_PACKAGES" \
-	"$DIST" "$VROOTDIR/$VHOST" "$MIRROR" ; then
-    echo "${0##*/}: error: debootstrap failure. Cannot continue."
+## use "vserver ... build" to build the new vserver
+if ! /usr/sbin/vserver "$VHOST" build -m debootstrap \
+       --rootdir "$VROOTDIR" --hostname "$VHOST" --interface "$INTERFACE:$IP" \
+       -- -d "$DIST" -m "$MIRROR" \
+       -- $ARCH_ARGUMENT \
+        --include="$INSTALL_PACKAGES" --exclude="$REMOVE_PACKAGES"
+  then
+    echo "${0##*/}: error: vserver-build failure. Cannot continue."
     exit 1
 fi
 
@@ -406,15 +406,8 @@ deb http://security.debian.org $DIST/updates main contrib non-free
 
 EOF
 
-# Fix up the available device nodes, for security
+# Fix up the available device nodes (mostly done by vserver-build above)
 if cd "$VROOTDIR/$VHOST/dev"; then
-    mkdir -p /var/lib/vserver-debiantools
-    tar cfp /var/lib/vserver-debiantools/dev.tar.$$ full null ptmx random tty urandom zero
-    rm -rf *
-    tar xfp /var/lib/vserver-debiantools/dev.tar.$$
-    rm -f /var/lib/vserver-debiantools/dev.tar.$$
-    mkdir pts
-    mkdir shm
     ln -s /proc/self/fd fd
     ln -s fd/2 stderr
     ln -s fd/0 stdin
@@ -429,7 +422,7 @@ cat << EOF > "$VROOTDIR/$VHOST/etc/hosts"
 # /etc/hosts
 
 127.0.0.1	localhost
-$IP	$VHOST.$VDOMAIN $VHOST
+$IP_ADDR	$VHOST.$VDOMAIN $VHOST
 
 # The following lines are desirable for IPv6 capable hosts
 # (added automatically by netbase upgrade)
@@ -452,6 +445,11 @@ if [ -f /etc/apt/apt.conf ]; then
     cp /etc/apt/apt.conf $VROOTDIR/$VHOST/etc/apt/apt.conf
 fi
 
+# support the apt.conf.d directories
+if [ -d /etc/apt/apt.conf.d ] ; then
+    cp -a /etc/apt/apt.conf.d/* $VROOTDIR/$VHOST/etc/apt/apt.conf.d/ > /dev/null 2>&1
+fi
+
 # This old uname was putting the name of the *host* into the vserver motd
 # uname -a > $VROOTDIR/$VHOST/etc/motd
 
@@ -467,45 +465,10 @@ cat << EOF > "$VROOTDIR/$VHOST/etc/fstab"
 proc		/proc		proc	defaults			0	0
 EOF
 
-# The new vserver patch now automatically mounts /proc
-# Debian needs /dev/pts mounting "gid=5,mode=620" by vserver
-cat << EOF > "$VROOTDIR/$VHOST/etc/mtab"
-/dev/hdv1 / vfs none 0 0
-proc /proc proc rw 0 0
-devpts /dev/pts devpts rw,gid=5,mode=620 0 0
-EOF
-
 # Create a reduced inittab that doesn't start getty on the consoles
-cat << EOF > "$VROOTDIR/$VHOST/etc/inittab"
-# /etc/inittab: init(8) configuration.
-
-# The default runlevel.
-id:2:initdefault:
-
-# Boot-time system configuration/initialization script.
-# This is run first except when booting in emergency (-b) mode.
-si::sysinit:/etc/init.d/rcS
-
-# What to do in single-user mode.
-~~:S:wait:/sbin/sulogin
-
-# /etc/init.d executes the S and K scripts upon change
-# of runlevel.
-#
-# Runlevel 0 is halt.
-# Runlevel 1 is single-user.
-# Runlevels 2-5 are multi-user.
-# Runlevel 6 is reboot.
-
-l0:0:wait:/etc/init.d/rc 0
-l1:1:wait:/etc/init.d/rc 1
-l2:2:wait:/etc/init.d/rc 2
-l3:3:wait:/etc/init.d/rc 3
-l4:4:wait:/etc/init.d/rc 4
-l5:5:wait:/etc/init.d/rc 5
-l6:6:wait:/etc/init.d/rc 6
-
-EOF
+mv "$VROOTDIR/$VHOST/etc/inittab" "$VROOTDIR/$VHOST/etc/inittab.dist" && \
+    grep -v respawn\:/sbin/getty "$VROOTDIR/$VHOST/etc/inittab.dist" > \
+    "$VROOTDIR/$VHOST/etc/inittab" && rm -f "$VROOTDIR/$VHOST/etc/inittab.dist"
 
 # By default the Debian Install script runs zillions of cron jobs at
 # 0625 every morning.  On a system with lots of vservers all trying to
@@ -549,22 +512,6 @@ $MONTHLY $HOUR	1 * *	root	test -e /usr/sbin/anacron || run-parts --report /etc/c
 
 EOF
 
-# Create default /etc/vservers entry
-cat << EOF > /etc/vservers/$VHOST.conf
-S_HOSTNAME="$VHOST"
-IPROOT="$IP"
-IPROOTDEV="$INTERFACE"
-ONBOOT="yes"
-S_NICE=""
-S_FLAGS="lock nproc$FAKEINIT"
-ULIMIT="-H -n 1024"
-S_CAPS="CAP_NET_RAW"
-
-# *NOT* DNS domain name, for NIS only
-S_DOMAINNAME=""
-
-EOF
-
 if [ -n "$EXTRA_PACKAGES" ]; then
     EXTRA_PACKAGES_INSTALL="apt-get --assume-yes install ${EXTRA_PACKAGES//,/ }"
 fi
@@ -595,12 +542,16 @@ dpkg -P `echo $REMOVE_PACKAGES | sed -e 's/,/ /g'`
 for link in $REMOVE_LINKS
 do
     update-rc.d -f \$link remove
+    update-rc.d -f \$link stop 90 3 .
 done
 
 $EXTRA_PACKAGES_INSTALL
 
 EOF
 
+## start vserver before we can exec anything inside it
+vserver $VHOST start
+
 # Run the above commands from within the server
 chmod 755 $VROOTDIR/$VHOST/vserver-config.sh
 vserver $VHOST exec /vserver-config.sh
@@ -622,30 +573,22 @@ if [ -f "$SSH_KEYS" ]; then
     chmod 600 "$VROOTDIR/$VHOST/root/.ssh/authorized_keys"
 fi
 
-# Install the vreboot/rebootmgr utility--hopefully this will disappear soon
-VREBOOT_LOCATION="/usr/lib/util-vserver/vreboot"
-if [ "$COPY_VREBOOT" == "true" -a -x "$VREBOOT_LOCATION" ]; then
-    cp -a "$VREBOOT_LOCATION" "$VROOTDIR/$VHOST/usr/local/sbin/"
-    chmod 750 "$VROOTDIR/$VHOST/usr/local/sbin/vreboot"
-    # ln -s "$VROOTDIR/$VHOST/usr/local/sbin/"{vshutdown,vreboot}
-    # ln -s "$VROOTDIR/$VHOST/usr/local/sbin/"{vhalt,vreboot}
-fi
-
-# Stop all the processes that were started inside the server
-export PREVLEVEL=2
-vserver $VHOST exec /etc/init.d/rc 0
+## stop the vserver
 vserver $VHOST stop
 
 # Populate the archive for future virtual servers
-if [ ! -d $VROOTDIR/ARCHIVES/$DIST ]; then
-    mkdir -p $VROOTDIR/ARCHIVES/$DIST
+if [ $PKGCACHE -eq 1 ] ; then
+    if [ ! -d "$VROOTDIR/ARCHIVES/$DIST/partial" ] ; then
+	mkdir -p "$VROOTDIR/ARCHIVES/$DIST/partial"
+    fi
+    cp "$VROOTDIR/$VHOST/"var/cache/apt/archives/*.deb "$VROOTDIR/ARCHIVES/$DIST"
+    echo "Cleanup package cache."
+    aptcleanup "$VROOTDIR/ARCHIVES/$DIST"
 fi
-cp $VROOTDIR/$VHOST/var/cache/apt/archives/*.deb $VROOTDIR/ARCHIVES/$DIST
 
-echo "Restarting rebootmgr."
-invoke-rc.d rebootmgr restart
 echo
-echo "You should now adjust /etc/vservers/$VHOST.conf to suit your needs,"
+echo "You should now adjust the configuration in /etc/vservers/$VHOST/"
+echo "to suit your needs,"
 echo "or else just go ahead and type \`vserver $VHOST start' to start"
 echo "your new virtual server.  debian/rules!"
 echo
