@@ -25,6 +25,8 @@ VERSION='0.3.2'
 # ----------------------------------------------------------------------
 # Configurable items:
 
+shopt -s extglob
+
 # Root directory of your virtual servers (probably shouldn't change this)
 VROOTDIR="/vservers"
 
@@ -45,8 +47,8 @@ REMOVE_PACKAGES="dhcp-client,lilo,makedev,pcmcia-cs,ppp,pppconfig,pppoe,pppoecon
 # sysvinit services relating to hardware access to remove
 REMOVE_LINKS="klogd hwclock.sh setserial urandom networking umountfs halt reboot"
 
-# Which debian distribution (warning: this has only been tested with woody)
-DIST="woody"
+# Which debian distribution (warning: this has only been tested with woody and sarge)
+DIST="sarge"
 
 # Default network interface for vservers:
 INTERFACE="eth0"
@@ -94,7 +96,7 @@ Options:
 				(autodetected on Debian host if dpkg available)
 	--copy-vreboot		install "vreboot/vshutdown/vhalt"
 	--no-copy-vreboot	don't install "vreboot/vshutdown/vhalt"
-	--dist			defaults to "woody", passed to debootstrap.
+	--dist			defaults to "sarge", passed to debootstrap.
 	--fakeinit		use "/sbin/init" to boot vserver
 	--conffile		extra configuration file to load.
 	--interface		interface for IP addresses (if not "eth0")
@@ -160,13 +162,13 @@ parse_args ()
 			case "$2" in
 			    [a-z]*)
 				DIST="$2"
-				if [ "woody" != "$2" ]; then
-				    echo "${0##*/} warning: I only know how to do \"woody\", be careful!" 1>&2
+				if [ "woody" != "$2" ] && [ "sarge" != "$2 ] ; then
+				    echo "${0##*/} warning: I only know how to do \"woody\" and \"sarge\", be careful!" 1>&2
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a Debian distribution" 1>&2
-				echo 'e.g. "woody"' 1>&2
+				echo 'e.g. "woody" or "sarge"' 1>&2
 				exit 1
 				;;
 			esac
@@ -192,7 +194,7 @@ parse_args ()
 			;;
 		    --hostname)
 			case "$2" in
-			    [a-z]*[a-z0-9])
+			    *([a-z0-9])[a-z]*([a-z0-9]))
 				VHOST="$2"
 				;;
 			    *)
